# cashfree-lrs-client
CashFree LRS APIs (v2)

This Python package is automatically generated by the [OpenAPI Generator](https://openapi-generator.tech) project:

- API version: 1.0.1
- Package version: 1.0.0
- Build package: org.openapitools.codegen.languages.PythonClientCodegen
For more information, please visit [https://docs.cashfree.com](https://docs.cashfree.com)

## Requirements.

Python &gt;&#x3D;3.7

## Migration from other generators like python and python-legacy

### Changes
1. This generator uses spec case for all (object) property names and parameter names.
    - So if the spec has a property name like camelCase, it will use camelCase rather than camel_case
    - So you will need to update how you input and read properties to use spec case
2. Endpoint parameters are stored in dictionaries to prevent collisions (explanation below)
    - So you will need to update how you pass data in to endpoints
3. Endpoint responses now include the original response, the deserialized response body, and (todo)the deserialized headers
    - So you will need to update your code to use response.body to access deserialized data
4. All validated data is instantiated in an instance that subclasses all validated Schema classes and Decimal/str/list/tuple/frozendict/NoneClass/BoolClass/bytes/io.FileIO
    - This means that you can use isinstance to check if a payload validated against a schema class
    - This means that no data will be of type None/True/False
        - ingested None will subclass NoneClass
        - ingested True will subclass BoolClass
        - ingested False will subclass BoolClass
        - So if you need to check is True/False/None, instead use instance.is_true_oapg()/.is_false_oapg()/.is_none_oapg()
5. All validated class instances are immutable except for ones based on io.File
    - This is because if properties were changed after validation, that validation would no longer apply
    - So no changing values or property values after a class has been instantiated
6. String + Number types with formats
    - String type data is stored as a string and if you need to access types based on its format like date,
    date-time, uuid, number etc then you will need to use accessor functions on the instance
    - type string + format: See .as_date_oapg, .as_datetime_oapg, .as_decimal_oapg, .as_uuid_oapg
    - type number + format: See .as_float_oapg, .as_int_oapg
    - this was done because openapi/json-schema defines constraints. string data may be type string with no format
    keyword in one schema, and include a format constraint in another schema
    - So if you need to access a string format based type, use as_date_oapg/as_datetime_oapg/as_decimal_oapg/as_uuid_oapg
    - So if you need to access a number format based type, use as_int_oapg/as_float_oapg
7. Property access on AnyType(type unset) or object(dict) schemas
    - Only required keys with valid python names are properties like .someProp and have type hints
    - All optional keys may not exist, so properties are not defined for them
    - One can access optional values with dict_instance['optionalProp'] and KeyError will be raised if it does not exist
    - Use get_item_oapg if you need a way to always get a value whether or not the key exists
        - If the key does not exist, schemas.unset is returned from calling dict_instance.get_item_oapg('optionalProp')
        - All required and optional keys have type hints for this method, and @typing.overload is used
        - A type hint is also generated for additionalProperties accessed using this method
    - So you will need to update you code to use some_instance['optionalProp'] to access optional property
    and additionalProperty values
8. The location of the api classes has changed
    - Api classes are located in your_package.apis.tags.some_api
    - This change was made to eliminate redundant code generation
    - Legacy generators generated the same endpoint twice if it had > 1 tag on it
    - This generator defines an endpoint in one class, then inherits that class to generate
      apis by tags and by paths
    - This change reduces code and allows quicker run time if you use the path apis
        - path apis are at your_package.apis.paths.some_path
    - Those apis will only load their needed models, which is less to load than all of the resources needed in a tag api
    - So you will need to update your import paths to the api classes

### Why are Oapg and _oapg used in class and method names?
Classes can have arbitrarily named properties set on them
Endpoints can have arbitrary operationId method names set
For those reasons, I use the prefix Oapg and _oapg to greatly reduce the likelihood of collisions
on protected + public classes/methods.
oapg stands for OpenApi Python Generator.

### Object property spec case
This was done because when payloads are ingested, they can be validated against N number of schemas.
If the input signature used a different property name then that has mutated the payload.
So SchemaA and SchemaB must both see the camelCase spec named variable.
Also it is possible to send in two properties, named camelCase and camel_case in the same payload.
That use case should be support so spec case is used.

### Parameter spec case
Parameters can be included in different locations including:
- query
- path
- header
- cookie

Any of those parameters could use the same parameter names, so if every parameter
was included as an endpoint parameter in a function signature, they would collide.
For that reason, each of those inputs have been separated out into separate typed dictionaries:
- query_params
- path_params
- header_params
- cookie_params

So when updating your code, you will need to pass endpoint parameters in using those
dictionaries.

### Endpoint responses
Endpoint responses have been enriched to now include more information.
Any response reom an endpoint will now include the following properties:
response: urllib3.HTTPResponse
body: typing.Union[Unset, Schema]
headers: typing.Union[Unset, TODO]
Note: response header deserialization has not yet been added


## Installation & Usage
### pip install

If the python package is hosted on a repository, you can install directly using:

```sh
pip install git+https://github.com/GIT_USER_ID/GIT_REPO_ID.git
```
(you may need to run `pip` with root permission: `sudo pip install git+https://github.com/GIT_USER_ID/GIT_REPO_ID.git`)

Then import the package:
```python
import cashfree_lrs_client
```

### Setuptools

Install via [Setuptools](http://pypi.python.org/pypi/setuptools).

```sh
python setup.py install --user
```
(or `sudo python setup.py install` to install the package for all users)

Then import the package:
```python
import cashfree_lrs_client
```

## Getting Started

Please follow the [installation procedure](#installation--usage) and then run the following:

```python

import time
import cashfree_lrs_client
from pprint import pprint
from cashfree_lrs_client.apis.tags import lrs_api
from cashfree_lrs_client.model.create_beneficiary_request import CreateBeneficiaryRequest
from cashfree_lrs_client.model.create_order_request import CreateOrderRequest
from cashfree_lrs_client.model.create_order_response import CreateOrderResponse
from cashfree_lrs_client.model.create_remitter_request import CreateRemitterRequest
from cashfree_lrs_client.model.error import Error
from cashfree_lrs_client.model.fetch_forex_rate_request import FetchForexRateRequest
from cashfree_lrs_client.model.setup_webhooks_request import SetupWebhooksRequest
from cashfree_lrs_client.model.success_message import SuccessMessage
from cashfree_lrs_client.model.upload_documents_request import UploadDocumentsRequest
# Defining the host is optional and defaults to https://sandbox.cashfree.com/pg/lrs
# See configuration.py for a list of all supported configuration parameters.
configuration = cashfree_lrs_client.Configuration(
    host = "https://sandbox.cashfree.com/pg/lrs"
)

# The client must configure the authentication and authorization parameters
# in accordance with the API server security policy.
# Examples for each auth method are provided below, use the example that
# satisfies your auth use case.

# Configure API key authorization: X-Client-ID
configuration.api_key['X-Client-ID'] = 'YOUR_API_KEY'

# Uncomment below to setup prefix (e.g. Bearer) for API key, if needed
# configuration.api_key_prefix['X-Client-ID'] = 'Bearer'

# Configure API key authorization: X-Client-Secret
configuration.api_key['X-Client-Secret'] = 'YOUR_API_KEY'

# Uncomment below to setup prefix (e.g. Bearer) for API key, if needed
# configuration.api_key_prefix['X-Client-Secret'] = 'Bearer'

# Configure API key authorization: X-API-Version
configuration.api_key['X-API-Version'] = 'YOUR_API_KEY'

# Uncomment below to setup prefix (e.g. Bearer) for API key, if needed
# configuration.api_key_prefix['X-API-Version'] = 'Bearer'

# Enter a context with an instance of the API client
with cashfree_lrs_client.ApiClient(configuration) as api_client:
    # Create an instance of the API class
    api_instance = lrs_api.LrsApi(api_client)
    body = dict(
        files=[
            open('/path/to/file', 'rb')
        ],
    ) # {str: (bool, date, datetime, dict, float, int, list, str, none_type)} |  (optional)

    try:
        # Upload Documents in Bulk
        api_instance.bulk_documents_upload(body=body)
    except cashfree_lrs_client.ApiException as e:
        print("Exception when calling LrsApi->bulk_documents_upload: %s\n" % e)
```

## Documentation for API Endpoints

All URIs are relative to *https://sandbox.cashfree.com/pg/lrs*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*LrsApi* | [**bulk_documents_upload**](docs/apis/tags/LrsApi.md#bulk_documents_upload) | **post** /orders/documents/upload | Upload Documents in Bulk
*LrsApi* | [**create_beneficiary**](docs/apis/tags/LrsApi.md#create_beneficiary) | **post** /beneficiaries | Create Beneficiary
*LrsApi* | [**create_order**](docs/apis/tags/LrsApi.md#create_order) | **post** /orders | Create LRS Order
*LrsApi* | [**create_remitter**](docs/apis/tags/LrsApi.md#create_remitter) | **post** /remitters | Create Remitter
*LrsApi* | [**fetch_forex_rate**](docs/apis/tags/LrsApi.md#fetch_forex_rate) | **post** /fx-rate/details | Fetch FX Rate
*LrsApi* | [**process_order**](docs/apis/tags/LrsApi.md#process_order) | **post** /orders/{order_id}/process | Process Order
*LrsApi* | [**setup_webhooks**](docs/apis/tags/LrsApi.md#setup_webhooks) | **post** /webhooks | Setup Webhooks
*LrsApi* | [**upload_documents**](docs/apis/tags/LrsApi.md#upload_documents) | **post** /orders/{order_id}/documents/upload | Upload Documents

## Documentation For Models

 - [CreateBeneficiaryRequest](docs/models/CreateBeneficiaryRequest.md)
 - [CreateOrderRequest](docs/models/CreateOrderRequest.md)
 - [CreateOrderResponse](docs/models/CreateOrderResponse.md)
 - [CreateRemitterRequest](docs/models/CreateRemitterRequest.md)
 - [Currency](docs/models/Currency.md)
 - [Error](docs/models/Error.md)
 - [FetchForexRateRequest](docs/models/FetchForexRateRequest.md)
 - [Purpose](docs/models/Purpose.md)
 - [SetupWebhooksRequest](docs/models/SetupWebhooksRequest.md)
 - [SuccessMessage](docs/models/SuccessMessage.md)

## Documentation For Authorization


## X-API-Version

- **Type**: API key
- **API key parameter name**: X-API-Version
- **Location**: HTTP header


## X-Client-ID

- **Type**: API key
- **API key parameter name**: X-Client-ID
- **Location**: HTTP header

 Authentication schemes defined for the API:
## X-Client-Secret

- **Type**: API key
- **API key parameter name**: X-Client-Secret
- **Location**: HTTP header


## Author

nextgenapi@cashfree.com

## Notes for Large OpenAPI documents
If the OpenAPI document is large, imports in cashfree_lrs_client.apis and cashfree_lrs_client.models may fail with a
RecursionError indicating the maximum recursion limit has been exceeded. In that case, there are a couple of solutions:

Solution 1:
Use specific imports for apis and models like:
- `from cashfree_lrs_client.apis.default_api import DefaultApi`
- `from cashfree_lrs_client.model.pet import Pet`

Solution 1:
Before importing the package, adjust the maximum recursion limit as shown below:
```
import sys
sys.setrecursionlimit(1500)
import cashfree_lrs_client
from cashfree_lrs_client.apis import *
from cashfree_lrs_client.models import *
```
